@@ -54,6 +54,7 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include <inttypes.h>
 
 #include "gkhash.h"
 
@@ -95,8 +96,6 @@ static GWSReader *gwsreader;
 static GDash *dash;
 /* Data holder structure */
 static GHolder *holder;
-/* Log line properties structure */
-static Logs *logs;
 /* Old signal mask */
 static sigset_t oldset;
 /* Curses windows */
@@ -107,21 +106,26 @@ static int main_win_height = 0;
 /* *INDENT-OFF* */
 static GScroll gscroll = {
   {
-    {0, 0}, /* visitors    {scroll, offset} */
-    {0, 0}, /* requests    {scroll, offset} */
-    {0, 0}, /* req static  {scroll, offset} */
-    {0, 0}, /* not found   {scroll, offset} */
-    {0, 0}, /* hosts       {scroll, offset} */
-    {0, 0}, /* os          {scroll, offset} */
-    {0, 0}, /* browsers    {scroll, offset} */
-    {0, 0}, /* visit times {scroll, offset} */
-    {0, 0}, /* referrers   {scroll, offset} */
-    {0, 0}, /* ref sites   {scroll, offset} */
-    {0, 0}, /* keywords    {scroll, offset} */
+     {0, 0}, /* VISITORS        { scroll, offset} */
+     {0, 0}, /* REQUESTS        { scroll, offset} */
+     {0, 0}, /* REQUESTS_STATIC { scroll, offset} */
+     {0, 0}, /* NOT_FOUND       { scroll, offset} */
+     {0, 0}, /* HOSTS           { scroll, offset} */
+     {0, 0}, /* OS              { scroll, offset} */
+     {0, 0}, /* BROWSERS        { scroll, offset} */
+     {0, 0}, /* VISIT_TIMES     { scroll, offset} */
+     {0, 0}, /* VIRTUAL_HOSTS   { scroll, offset} */
+     {0, 0}, /* REFERRERS       { scroll, offset} */
+     {0, 0}, /* REFERRING_SITES { scroll, offset} */
+     {0, 0}, /* KEYPHRASES      { scroll, offset} */
+     {0, 0}, /* STATUS_CODES    { scroll, offset} */
+     {0, 0}, /* REMOTE_USER     { scroll, offset} */
+     {0, 0}, /* CACHE_STATUS    { scroll, offset} */
 #ifdef HAVE_GEOLOCATION
-    {0, 0}, /* geolocation {scroll, offset} */
+     {0, 0}, /* GEO_LOCATION    { scroll, offset} */
 #endif
-    {0, 0}, /* status      {scroll, offset} */
+     {0, 0}, /* MIME_TYPE       { scroll, offset} */
+     {0, 0}, /* TLS_TYPE        { scroll, offset} */
   },
   0,         /* current module */
   0,         /* main dashboard scroll */
@@ -163,9 +167,6 @@ house_keeping (void) {
   geoip_free ();
 #endif
 
-  /* LOGGER */
-  free_logs (logs);
-
   /* INVALID REQUESTS */
   if (conf.invalid_requests_log) {
     LOG_DEBUG (("Closing invalid requests log.\n"));
@@ -175,7 +176,7 @@ house_keeping (void) {
   /* UNKNOWNS */
   if (conf.unknowns_log) {
     LOG_DEBUG (("Closing unknowns log.\n"));
-    unknowns_log_close();
+    unknowns_log_close ();
   }
 
   /* CONFIGURATION */
@@ -211,7 +212,7 @@ cleanup (int ret) {
 
   /* unable to process valid data */
   if (ret)
-    output_logerrors (logs);
+    output_logerrors ();
 
   house_keeping ();
 }
@@ -375,25 +376,32 @@ allocate_data (void) {
   }
 }
 
+static void
+clean_stdscrn (void) {
+  int row, col;
+
+  getmaxyx (stdscr, row, col);
+  draw_header (stdscr, "", "%s", row - 1, 0, col, color_default);
+}
+
 /* A wrapper to render all windows within the dashboard. */
 static void
-render_screens (void) {
+render_screens (uint32_t offset) {
   GColors *color = get_color (COLOR_DEFAULT);
-  int row, col, chg = 0;
+  int row, col;
   char time_str_buf[32];
 
   getmaxyx (stdscr, row, col);
   term_size (main_win, &main_win_height);
 
   generate_time ();
-  strftime (time_str_buf, sizeof (time_str_buf), "%a %b %e %T %Y", &now_tm);
-  chg = *logs->processed - logs->offset;
+  strftime (time_str_buf, sizeof (time_str_buf), "%d/%b/%Y:%T", &now_tm);
 
   draw_header (stdscr, "", "%s", row - 1, 0, col, color_default);
 
   wattron (stdscr, color->attr | COLOR_PAIR (color->pair->idx));
   mvaddstr (row - 1, 1, T_HELP_ENTER);
-  mvprintw (row - 1, col / 2 - 10, "%d - %s", chg, time_str_buf);
+  mvprintw (row - 1, col / 2 - 10, "%" PRIu32 "/r - %s", offset, time_str_buf);
   mvaddstr (row - 1, col - 6 - strlen (T_QUIT), T_QUIT);
   mvprintw (row - 1, col - 5, "%s", GO_VERSION);
   wattroff (stdscr, color->attr | COLOR_PAIR (color->pair->idx));
@@ -411,16 +419,17 @@ render_screens (void) {
 }
 
 /* Collapse the current expanded module */
-static void
+static int
 collapse_current_module (void) {
   if (!gscroll.expanded)
-    return;
+    return 1;
 
   gscroll.expanded = 0;
   reset_scroll_offsets (&gscroll);
   free_dashboard (dash);
   allocate_data ();
-  render_screens ();
+
+  return 0;
 }
 
 /* Display message a the bottom of the terminal dashboard that panel
@@ -435,11 +444,11 @@ disabled_panel_msg (GModule module) {
 }
 
 /* Set the current module/panel */
-static void
+static int
 set_module_to (GScroll * scrll, GModule module) {
   if (get_module_index (module) == -1) {
     disabled_panel_msg (module);
-    return;
+    return 1;
   }
 
   /* scroll to panel */
@@ -449,7 +458,7 @@ set_module_to (GScroll * scrll, GModule module) {
   /* reset expanded module */
   collapse_current_module ();
   scrll->current = module;
-  render_screens ();
+  return 0;
 }
 
 /* Scroll expanded module or terminal dashboard to the top */
@@ -514,14 +523,14 @@ expand_current_module (void) {
 }
 
 /* Expand the clicked module/panel given the Y event coordinate. */
-static void
+static int
 expand_module_from_ypos (int y) {
   /* ignore header/footer clicks */
   if (y < MAX_HEIGHT_HEADER || y == LINES - 1)
-    return;
+    return 1;
 
   if (set_module_from_mouse_event (&gscroll, dash, y))
-    return;
+    return 1;
 
   reset_scroll_offsets (&gscroll);
   gscroll.expanded = 1;
@@ -531,21 +540,22 @@ expand_module_from_ypos (int y) {
   allocate_holder_by_module (gscroll.current);
   allocate_data ();
 
-  render_screens ();
+  return 0;
 }
 
 /* Expand the clicked module/panel */
-static void
+static int
 expand_on_mouse_click (void) {
   int ok_mouse;
   MEVENT event;
 
   ok_mouse = getmouse (&event);
   if (!conf.mouse_support || ok_mouse != OK)
-    return;
+    return 1;
 
   if (event.bstate & BUTTON1_CLICKED)
-    expand_module_from_ypos (event.y);
+    return expand_module_from_ypos (event.y);
+  return 1;
 }
 
 /* Scroll dowm expanded module to the last row */
@@ -636,34 +646,35 @@ page_down_module (void) {
 
 /* Create a new find dialog window and render it. Upon closing the
  * window, dashboard is refreshed. */
-static void
+static int
 render_search_dialog (int search) {
   if (render_find_dialog (main_win, &gscroll))
-    return;
+    return 1;
 
   pthread_mutex_lock (&gdns_thread.mutex);
   search = perform_next_find (holder, &gscroll);
   pthread_mutex_unlock (&gdns_thread.mutex);
   if (search != 0)
-    return;
+    return 1;
 
   free_dashboard (dash);
   allocate_data ();
-  render_screens ();
+
+  return 0;
 }
 
 /* Search for the next occurrence within the dashboard structure */
-static void
+static int
 search_next_match (int search) {
   pthread_mutex_lock (&gdns_thread.mutex);
   search = perform_next_find (holder, &gscroll);
   pthread_mutex_unlock (&gdns_thread.mutex);
   if (search != 0)
-    return;
+    return 1;
 
   free_dashboard (dash);
   allocate_data ();
-  render_screens ();
+  return 0;
 }
 
 /* Update holder structure and dashboard screen */
@@ -679,7 +690,6 @@ tail_term (void) {
   allocate_data ();
 
   term_size (main_win, &main_win_height);
-  render_screens ();
 }
 
 static void
@@ -793,7 +803,7 @@ verify_inode (FILE * fp, GLog * glog) {
 }
 
 /* Process appended log data */
-static void
+static int
 perform_tail_follow (GLog * glog) {
   FILE *fp = NULL;
   char buf[READ_BYTES + 1] = { 0 };
@@ -804,13 +814,11 @@ perform_tail_follow (GLog * glog) {
     parse_tail_follow (glog, glog->pipe);
     /* did we read something from the pipe? */
     if (0 == glog->bytes)
-      return;
+      return 1;
 
     glog->length += glog->bytes;
     goto out;
   }
-  if (logs->load_from_disk_only)
-    return;
 
   length = file_size (glog->filename);
 
@@ -818,7 +826,7 @@ perform_tail_follow (GLog * glog) {
   /* ###NOTE: This assumes the log file being read can be of smaller size, e.g.,
    * rotated/truncated file or larger when data is appended */
   if (length == glog->length)
-    return;
+    return 1;
 
   if (!(fp = fopen (glog->filename, "r")))
     FATAL ("Unable to read the specified log file '%s'. %s", glog->filename, strerror (errno));
@@ -854,29 +862,42 @@ perform_tail_follow (GLog * glog) {
 out:
 
   if (!conf.output_stdout) {
-    struct timespec ts = {.tv_sec = 0,.tv_nsec = 200000000 };   /* 0.2 seconds */
-
     tail_term ();
-    if (nanosleep (&ts, NULL) == -1 && errno != EINTR)
-      FATAL ("nanosleep: %s", strerror (errno));
   } else {
     tail_html ();
   }
+
+  return 0;
+}
+
+/* Loop over and perform a follow for the given logs */
+static void
+tail_loop_html (Logs * logs) {
+  struct timespec refresh = {
+    .tv_sec = conf.html_refresh ? conf.html_refresh : HTML_REFRESH,
+    .tv_nsec = 0,
+  };
+  int i = 0;
+
+  while (1) {
+    if (conf.stop_processing)
+      break;
+
+    for (i = 0; i < logs->size; ++i)
+      perform_tail_follow (&logs->glog[i]);     /* 0.2 secs */
+    if (nanosleep (&refresh, NULL) == -1 && errno != EINTR)
+      FATAL ("nanosleep: %s", strerror (errno));
+  }
 }
 
 /* Entry point to start processing the HTML output */
 static void
-process_html (const char *filename) {
-  int i = 0;
-  struct timespec refresh = {
-    .tv_sec = conf.html_refresh ? conf.html_refresh : HTML_REFRESH,
-    .tv_nsec = 0,
-  };
-
+process_html (Logs * logs, const char *filename) {
   /* render report */
   pthread_mutex_lock (&gdns_thread.mutex);
   output_html (holder, filename);
   pthread_mutex_unlock (&gdns_thread.mutex);
+
   /* not real time? */
   if (!conf.real_time_html)
     return;
@@ -893,15 +914,7 @@ process_html (const char *filename) {
     return;
 
   set_ready_state ();
-  while (1) {
-    if (conf.stop_processing)
-      break;
-
-    for (i = 0; i < logs->size; ++i)
-      perform_tail_follow (&logs->glog[i]);     /* 0.2 secs */
-    if (nanosleep (&refresh, NULL) == -1 && errno != EINTR)
-      FATAL ("nanosleep: %s", strerror (errno));
-  }
+  tail_loop_html (logs);
   close (gwswriter->fd);
 }
 
@@ -945,7 +958,6 @@ window_resize (void) {
   werase (stdscr);
   term_size (main_win, &main_win_height);
   refresh ();
-  render_screens ();
 }
 
 /* Create a new sort dialog window and render it. Upon closing the
@@ -962,18 +974,37 @@ render_sort_dialog (void) {
   free_dashboard (dash);
   allocate_holder ();
   allocate_data ();
-  render_screens ();
+}
+
+static void
+term_tail_logs (Logs * logs) {
+  struct timespec ts = {.tv_sec = 0,.tv_nsec = 200000000 };     /* 0.2 seconds */
+  uint32_t offset = 0;
+  int i;
+
+  for (i = 0; i < logs->size; ++i) {
+    if (perform_tail_follow (&logs->glog[i]) == 0) {
+      offset = *logs->processed - logs->offset;
+      render_screens (offset);
+    }
+    if (nanosleep (&ts, NULL) == -1 && errno != EINTR) {
+      FATAL ("nanosleep: %s", strerror (errno));
+    }
+  }
 }
 
 /* Interfacing with the keyboard */
 static void
-get_keys (void) {
+get_keys (Logs * logs) {
   int search = 0;
-  int c, quit = 1, i;
+  int c, quit = 1;
+  uint32_t offset = 0;
 
   while (quit) {
     if (conf.stop_processing)
       break;
+
+    offset = *logs->processed - logs->offset;
     c = wgetch (stdscr);
     switch (c) {
     case 'q':  /* quit */
@@ -981,91 +1012,118 @@ get_keys (void) {
         quit = 0;
         break;
       }
-      collapse_current_module ();
+      if (collapse_current_module ())
+        render_screens (offset);
       break;
     case KEY_F (1):
     case '?':
     case 'h':
       load_help_popup (main_win);
-      render_screens ();
+      render_screens (offset);
       break;
     case 49:   /* 1 */
       /* reset expanded module */
-      set_module_to (&gscroll, VISITORS);
+      if (set_module_to (&gscroll, VISITORS) == 0)
+        render_screens (offset);
       break;
     case 50:   /* 2 */
       /* reset expanded module */
-      set_module_to (&gscroll, REQUESTS);
+      if (set_module_to (&gscroll, REQUESTS) == 0)
+        render_screens (offset);
       break;
     case 51:   /* 3 */
       /* reset expanded module */
-      set_module_to (&gscroll, REQUESTS_STATIC);
+      if (set_module_to (&gscroll, REQUESTS_STATIC) == 0)
+        render_screens (offset);
       break;
     case 52:   /* 4 */
       /* reset expanded module */
-      set_module_to (&gscroll, NOT_FOUND);
+      if (set_module_to (&gscroll, NOT_FOUND) == 0)
+        render_screens (offset);
       break;
     case 53:   /* 5 */
       /* reset expanded module */
-      set_module_to (&gscroll, HOSTS);
+      if (set_module_to (&gscroll, HOSTS) == 0)
+        render_screens (offset);
       break;
     case 54:   /* 6 */
       /* reset expanded module */
-      set_module_to (&gscroll, OS);
+      if (set_module_to (&gscroll, OS) == 0)
+        render_screens (offset);
       break;
     case 55:   /* 7 */
       /* reset expanded module */
-      set_module_to (&gscroll, BROWSERS);
+      if (set_module_to (&gscroll, BROWSERS) == 0)
+        render_screens (offset);
       break;
     case 56:   /* 8 */
       /* reset expanded module */
-      set_module_to (&gscroll, VISIT_TIMES);
+      if (set_module_to (&gscroll, VISIT_TIMES) == 0)
+        render_screens (offset);
       break;
     case 57:   /* 9 */
       /* reset expanded module */
-      set_module_to (&gscroll, VIRTUAL_HOSTS);
+      if (set_module_to (&gscroll, VIRTUAL_HOSTS) == 0)
+        render_screens (offset);
       break;
     case 48:   /* 0 */
       /* reset expanded module */
-      set_module_to (&gscroll, REFERRERS);
+      if (set_module_to (&gscroll, REFERRERS) == 0)
+        render_screens (offset);
       break;
     case 33:   /* shift + 1 */
       /* reset expanded module */
-      set_module_to (&gscroll, REFERRING_SITES);
+      if (set_module_to (&gscroll, REFERRING_SITES) == 0)
+        render_screens (offset);
       break;
-    case 34:   /* shift + 2 */
+    case 64:   /* shift + 2 */
       /* reset expanded module */
-      set_module_to (&gscroll, KEYPHRASES);
+      if (set_module_to (&gscroll, KEYPHRASES) == 0)
+        render_screens (offset);
       break;
     case 35:   /* Shift + 3 */
       /* reset expanded module */
-      set_module_to (&gscroll, STATUS_CODES);
+      if (set_module_to (&gscroll, STATUS_CODES) == 0)
+        render_screens (offset);
       break;
-    case 36:   /* Shift + 3 */
+    case 36:   /* Shift + 4 */
       /* reset expanded module */
-      set_module_to (&gscroll, REMOTE_USER);
+      if (set_module_to (&gscroll, REMOTE_USER) == 0)
+        render_screens (offset);
+      break;
+    case 37:   /* Shift + 5 */
+      /* reset expanded module */
+      if (set_module_to (&gscroll, CACHE_STATUS) == 0)
+        render_screens (offset);
       break;
 #ifdef HAVE_GEOLOCATION
-    case 37:   /* Shift + 4 */
+    case 94:   /* Shift + 6 */
       /* reset expanded module */
-      set_module_to (&gscroll, GEO_LOCATION);
+      if (set_module_to (&gscroll, GEO_LOCATION) == 0)
+        render_screens (offset);
       break;
 #endif
-    case 38:   /* Shift + 5 */
+    case 38:   /* Shift + 7 */
       /* reset expanded module */
-      set_module_to (&gscroll, CACHE_STATUS);
+      if (set_module_to (&gscroll, MIME_TYPE) == 0)
+        render_screens (offset);
+      break;
+    case 42:   /* Shift + 8 */
+      /* reset expanded module */
+      if (set_module_to (&gscroll, TLS_TYPE) == 0)
+        render_screens (offset);
       break;
     case 9:    /* TAB */
       /* reset expanded module */
       collapse_current_module ();
       if (next_module () == 0)
-        render_screens ();
+        render_screens (offset);
       break;
     case 353:  /* Shift TAB */
       /* reset expanded module */
       collapse_current_module ();
       if (previous_module () == 0)
-        render_screens ();
+        render_screens (offset);
       break;
     case 'g':  /* g = top */
       scroll_to_first_line ();
@@ -1093,7 +1151,8 @@ get_keys (void) {
       }
       break;
     case KEY_MOUSE:    /* handles mouse events */
-      expand_on_mouse_click ();
+      if (expand_on_mouse_click () == 0)
+        render_screens (offset);
       break;
     case 106:  /* j - DOWN expanded module */
       scroll_down_expanded_module ();
@@ -1121,10 +1180,12 @@ get_keys (void) {
       display_content (main_win, dash, &gscroll);
       break;
     case 'n':
-      search_next_match (search);
+      if (search_next_match (search) == 0)
+        render_screens (offset);
       break;
     case '/':
-      render_search_dialog (search);
+      if (render_search_dialog (search) == 0)
+        render_screens (offset);
       break;
     case 99:   /* c */
       if (conf.no_color)
@@ -1133,18 +1194,21 @@ get_keys (void) {
       free_dashboard (dash);
       allocate_data ();
       set_wbkgd (main_win, header_win);
-      render_screens ();
+      render_screens (offset);
       break;
     case 115:  /* s */
       render_sort_dialog ();
+      render_screens (offset);
       break;
     case 269:
     case KEY_RESIZE:
       window_resize ();
+      render_screens (offset);
       break;
     default:
-      for (i = 0; i < logs->size; ++i)
-        perform_tail_follow (&logs->glog[i]);
+      if (logs->load_from_disk_only)
+        break;
+      term_tail_logs (logs);
       break;
     }
   }
@@ -1170,12 +1234,13 @@ init_processing (void) {
   /* initialize storage */
   parsing_spinner->label = "SETTING UP STORAGE";
   init_storage ();
+  insert_methods_protocols ();
   set_spec_date_format ();
 }
 
 /* Determine the type of output, i.e., JSON, CSV, HTML */
 static void
-standard_output (void) {
+standard_output (Logs * logs) {
   char *csv = NULL, *json = NULL, *html = NULL;
 
   /* CSV */
@@ -1186,7 +1251,7 @@ standard_output (void) {
     output_json (holder, json);
   /* HTML */
   if (find_output_type (&html, "html", 1) == 0 || conf.output_format_idx == 0)
-    process_html (html);
+    process_html (logs, html);
 
   free (csv);
   free (html);
@@ -1195,14 +1260,15 @@ standard_output (void) {
 
 /* Output to a terminal */
 static void
-curses_output (void) {
+curses_output (Logs * logs) {
   allocate_data ();
   if (!conf.skip_term_resolver)
     gdns_thread_create ();
 
-  render_screens ();
+  clean_stdscrn ();
+  render_screens (0);
   /* will loop in here */
-  get_keys ();
+  get_keys (logs);
 }
 
 /* Set locale */
@@ -1358,10 +1424,11 @@ block_thread_signals (void) {
 }
 
 /* Initialize various types of data. */
-static void
+static Logs *
 initializer (void) {
   int i;
   FILE *pipe = NULL;
+  Logs *logs;
 
   /* drop permissions right away */
   if (conf.username)
@@ -1397,7 +1464,9 @@ initializer (void) {
 
   /* init random number generator */
   srand (getpid ());
-  init_pre_storage ();
+  init_pre_storage (logs);
+
+  return logs;
 }
 
 static char *
@@ -1464,7 +1533,7 @@ set_standard_output (void) {
 
 /* Set up curses. */
 static void
-set_curses (int *quit) {
+set_curses (Logs * logs, int *quit) {
   const char *err_log = NULL;
 
   setup_thread_signals ();
@@ -1498,6 +1567,7 @@ set_curses (int *quit) {
 /* Where all begins... */
 int
 main (int argc, char **argv) {
+  Logs *logs = NULL;
   int quit = 0, ret = 0;
 
   block_thread_signals ();
@@ -1508,7 +1578,7 @@ main (int argc, char **argv) {
   parse_conf_file (&argc, &argv);
   parse_cmd_line (argc, argv);
 
-  initializer ();
+  logs = initializer ();
 
   /* ignore outputting, process only */
   if (conf.process_and_exit) {
@@ -1519,7 +1589,7 @@ main (int argc, char **argv) {
   }
   /* set curses */
   else {
-    set_curses (&quit);
+    set_curses (logs, &quit);
   }
 
   /* no log/date/time format set */
@@ -1541,6 +1611,7 @@ main (int argc, char **argv) {
     goto clean;
   logs->offset = *logs->processed;
 
+  parsing_spinner->label = "RENDERING";
   /* init reverse lookup thread */
   gdns_init ();
   parse_initial_sort ();
@@ -1554,11 +1625,11 @@ main (int argc, char **argv) {
   }
   /* stdout */
   else if (conf.output_stdout) {
-    standard_output ();
+    standard_output (logs);
   }
   /* curses */
   else {
-    curses_output ();
+    curses_output (logs);
   }
 
   /* clean */
