@@ -202,7 +202,7 @@ read_fifo (GWSReader * gwsreader, void (*f) (int)) {
     (*pa) = xcalloc (1, sizeof (WSPacket));
     (*pa)->type = type;
     (*pa)->size = size;
-    (*pa)->data = xcalloc (size, sizeof (char));
+    (*pa)->data = xcalloc (size + 1, sizeof (char));
   }
 
   readh = (*pa)->len;   /* read from payload so far */
@@ -343,6 +343,8 @@ set_ws_opts (void) {
   ws_set_config_strict (1);
   if (conf.addr)
     ws_set_config_host (conf.addr);
+  if (conf.unix_socket)
+    ws_set_config_unix_socket (conf.unix_socket);
   if (conf.fifo_in)
     ws_set_config_pipein (conf.fifo_in);
   if (conf.fifo_out)
